/* 
    CMC RPC Client library
    Copyright (C) 2010 Dell, Inc

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include <string.h>
#include <stdlib.h>
#include <curl/curl.h>
#include <jansson.h>
#include "cmcrpc-c.h"

typedef struct cmcrpc_curl_baton_t {
    char *data;
    int size;
} cmcrpc_curl_baton_t;

struct cmcrpc_t {
    CURL *ch;
    cmcrpc_curl_baton_t baton;
};

static size_t 
cmcrpc_write_function(void *ptr, size_t size, size_t nmemb, void *stream)
{
    cmcrpc_curl_baton_t *baton = stream;
    size_t len = size * nmemb;

    if (len > baton->size) {
        baton->data = realloc(baton->data, baton->size + len + 1);
    }

    if (baton->data) {
        memcpy(&(baton->data[baton->size]), ptr, len);
        baton->size += len;
        baton->data[baton->size] = '\0';
    }

    return len;
}

void 
cmcrpc_init()
{
    curl_global_init(CURL_GLOBAL_NOTHING);
}

cmcrpc_t* 
cmcrpc_new(int flags, char *json)
{
    cmcrpc_t *rpc;
    struct curl_slist *headers=NULL;

    rpc = calloc(1, sizeof(*rpc));

    headers = curl_slist_append(headers, "Content-type: application/json"); 

    rpc->ch = curl_easy_init();
    curl_easy_setopt(rpc->ch, CURLOPT_URL, "http://127.0.0.1/api");
    curl_easy_setopt(rpc->ch, CURLOPT_NOPROGRESS, 1);
    curl_easy_setopt(rpc->ch, CURLOPT_FAILONERROR, 1);
    //curl_easy_setopt(rpc->ch, CURLOPT_VERBOSE, 1);
    curl_easy_setopt(rpc->ch, CURLOPT_POST, 1);
    curl_easy_setopt(rpc->ch, CURLOPT_PORT, 10351);
    curl_easy_setopt(rpc->ch, CURLOPT_CONNECTTIMEOUT, 5);
    curl_easy_setopt(rpc->ch, CURLOPT_HTTPHEADER, headers);
    curl_easy_setopt(rpc->ch, CURLOPT_POSTFIELDS, json);
    curl_easy_setopt(rpc->ch, CURLOPT_POSTFIELDSIZE, strlen(json));
    curl_easy_setopt(rpc->ch, CURLOPT_WRITEDATA, &rpc->baton);
    curl_easy_setopt(rpc->ch, CURLOPT_WRITEFUNCTION, cmcrpc_write_function);

    return rpc;
}

char* 
cmcrpc_data(cmcrpc_t *cr)
{
    return cr->baton.data;
}

void 
cmcrpc_free(cmcrpc_t *rpc)
{
    if (!rpc) return;
    if (rpc->ch) curl_easy_cleanup(rpc->ch);
    free(rpc->baton.data);
    free(rpc);
} 

int 
cmcrpc_run(cmcrpc_t *cr)
{
    return curl_easy_perform(cr->ch);
}

